/*
 * lcd.h
 *
 *  Created on: Aug 3, 2016
 *      Author: sebastian
 */

#ifndef LCD_H_
#define LCD_H_

#define MAX_LCDS        8

#ifdef __cplusplus
extern "C" {
#endif

extern void lcdHome        (const int fd) ;
extern void lcdClear       (const int fd) ;
extern void lcdDisplay     (const int fd, int state) ;
extern void lcdCursor      (const int fd, int state) ;
extern void lcdCursorBlink (const int fd, int state) ;
extern void lcdSendCommand (const int fd, unsigned char command) ;
extern void lcdPosition    (const int fd, int x, int y) ;
extern void lcdCharDef     (const int fd, int index, unsigned char data [8]) ;
extern void lcdPutchar     (const int fd, unsigned char data) ;
extern void lcdPuts        (const int fd, const char *string) ;
extern void lcdPrintf      (const int fd, const char *message, ...) ;

extern int  lcdInitI (const int rows, const int cols, const int bits,
        const int rs, const int strb) ;

extern int  lcdInit (void) ;

#ifdef __cplusplus
}
#endif

// Pinbelegung für das LCD, an verwendete Pins anpassen

#define LCD_RW		24
#define LCD_RS		27
#define LCD_EN		23




#endif /* LCD_H_ */
